from typing import (
    Type,
)

from educommon import (
    logger,
)

from edu_rdm_integration.adapters.runners import (
    WebEduRunner,
)
from edu_rdm_integration.consts import (
    LOGS_DELIMITER,
)
from edu_rdm_integration.export_data.base.helpers import (
    BaseExportDataRunnerHelper,
)
from edu_rdm_integration.export_data.base.results import (
    BaseExportDataRunnerResult,
)
from edu_rdm_integration.export_data.base.validators import (
    BaseExportDataRunnerValidator,
)


class BaseExportDataRunner(WebEduRunner):
    """
    Базовый класс ранеров функций выгрузки данных для интеграции с "Региональная витрина данных".
    """

    def __init__(self, *args, is_force_run: bool = False, **kwargs):
        self._is_force_run = is_force_run

        super().__init__(*args, **kwargs)

    def _prepare_helper_class(self) -> Type[BaseExportDataRunnerHelper]:
        """
        Возвращает класс помощника ранера функции.
        """
        return BaseExportDataRunnerHelper

    def _prepare_validator_class(self) -> Type[BaseExportDataRunnerValidator]:
        """
        Возвращает класс валидатора ранера функции.
        """
        return BaseExportDataRunnerValidator

    def _prepare_result_class(self) -> Type[BaseExportDataRunnerResult]:
        """
        Возвращает класс результата ранера функции.
        """
        return BaseExportDataRunnerResult

    def _prepare_model_ids_chunks(self, *args, model_ids_map=None, **kwargs):
        """
        Формирование чанков идентификаторов записей моделей для дальнейшей работы в рамках функций.
        """
        # model_ids_chunks = make_chunks(
        #     iterable=model_ids,
        #     size=settings.RDM_EXPORT_CHUNK_SIZE,
        # )

        return ()

    def _populate_queue_by_runnable_classes(self, *args, **kwargs):
        """
        Заполнение очереди запускаемыми объектами.
        """
        model_ids_chunks = self._prepare_model_ids_chunks(*args, **kwargs)

        for chunk_index, model_ids_chunk in enumerate(model_ids_chunks, start=1):
            runnable_classes = self._prepare_runnable_classes()

            model_ids_chunk = list(model_ids_chunk)

            for runnable_class in runnable_classes:
                runnable = runnable_class(model_ids=model_ids_chunk, *args, **kwargs)

                # TODO EDUSCHL-20274 Реализация форсированного выполнения функций
                if self._is_force_run:
                    logger.info(
                        f'{LOGS_DELIMITER * 2}force run {runnable_class.__name__} with logs chunk {chunk_index}..'
                    )
                    self.force_run(runnable=runnable, *args, **kwargs)
                else:

                    logger.info(
                        f'{LOGS_DELIMITER * 2}enqueue {runnable_class.__name__} with logs chunk {chunk_index}..'
                    )

                    self.enqueue(runnable=runnable, *args, **kwargs)

    # TODO EDUSCHL-20274 Реализация форсированного выполнения функций
    def force_run(self, runnable, *args, **kwargs):
        """Принудительное выполнение запускаемого объекта."""
        self.before_validate()
        self.validate()
        self.after_validate()

        if self.result.has_not_errors:
            runnable.before_validate()
            runnable.validate()
            runnable.after_validate()

            runnable.before_run(*args, **kwargs)
            runnable.run(*args, **kwargs)
            runnable.after_run(*args, **kwargs)

            self.result.append_entity(runnable.result)


from typing import (
    Dict,
    Tuple,
)

from educommon.integration_entities.enums import (
    EntityLogOperation,
)


# Перечень отслеживаемых моделей и перечней полей по операциям лога. Расширяется в продуктах
MODEL_FIELDS_LOG_FILTER: Dict[EntityLogOperation, Dict[str, Tuple]] = {
    EntityLogOperation.CREATE: {},
    EntityLogOperation.UPDATE: {},
    EntityLogOperation.DELETE: {}
}



from datetime import (
    date,
    datetime,
    time,
)
from typing import (
    TYPE_CHECKING,
)

from django.core.management.base import (
    BaseCommand,
)

from edu_rdm_integration.collect_data.collect import (
    BaseCollectModelsData,
)
from edu_rdm_integration.consts import (
    DATETIME_FORMAT,
)
from edu_rdm_integration.export_data.export import (
    BaseExportEntitiesData,
)
from edu_rdm_integration.models import (
    RegionalDataMartEntityEnum,
    RegionalDataMartModelEnum,
)


if TYPE_CHECKING:
    from django.core.management.base import (
        CommandParser,
    )


class BaseCollectModelDataCommand(BaseCommand):
    """
    Базовая команда для выполнения сбора данных моделей РВД.
    """

    def add_arguments(self, parser: 'CommandParser'):
        """
        Добавление параметров.
        """
        models = ', '.join([
            f'{key} - {value.title}'
            for key, value in RegionalDataMartModelEnum.get_enum_data().items()
        ])
        models_help_text = (
            f'Значением параметра является перечисление моделей РВД, для которых должен быть произведен сбор данных. '
            f'Перечисление моделей:\n{models}. Если модели не указываются, то сбор данных производится для всех '
            f'моделей. Модели перечисляются через запятую без пробелов.'
        )
        parser.add_argument(
            '--models',
            action='store',
            dest='models',
            type=lambda e: e.split(','),
            help=models_help_text,
        )

        parser.add_argument(
            '--logs_period_started_at',
            action='store',
            dest='logs_period_started_at',
            type=lambda started_at: datetime.strptime(started_at, DATETIME_FORMAT),
            default=datetime.combine(date.today(), time.min),
            help=(
                'Дата и время начала периода обрабатываемых логов. Значение предоставляется в формате '
                '"дд.мм.гггг чч:мм:сс". По умолчанию, сегодняшний день, время 00:00:00.'
            ),
        )

        parser.add_argument(
            '--logs_period_ended_at',
            action='store',
            dest='logs_period_ended_at',
            type=lambda ended_at: datetime.strptime(ended_at, DATETIME_FORMAT),
            default=datetime.combine(date.today(), time.max),
            help=(
                'Дата и время конца периода обрабатываемых логов. Значение предоставляется в формате '
                '"дд.мм.гггг чч:мм:сс". По умолчанию, сегодняшний день, время 23:59:59.'
            ),
        )

    def _prepare_collect_models_data_class(self, *args, **kwargs) -> BaseCollectModelsData:
        """Возвращает объект класса сбора данных моделей РВД."""
        raise NotImplementedError

    def handle(self, *args, **kwargs):
        """
        Выполнение действий команды.
        """
        collect_models_data = self._prepare_collect_models_data_class(*args, **kwargs)
        collect_models_data.collect()


class BaseExportEntityDataCommand(BaseCommand):
    """
    Базовая команда для выполнения выгрузки данных сущностей РВД за указанный период.
    """

    def add_arguments(self, parser: 'CommandParser'):
        """
        Добавление параметров.
        """
        entities = ', '.join([
            f'{key} - {value.title}'
            for key, value in RegionalDataMartEntityEnum.get_enum_data().items()
        ])
        entities_help_text = (
            f'Значением параметра является перечисление сущностей РВД, для которых должена быть произведена выгрузка '
            f'данных. Перечисление сущностей:\n{entities}. Если сущности не указываются, то выгрузка данных '
            f'производится для всех сущностей. Сущности перечисляются через запятую без пробелов.'
        )
        parser.add_argument(
            '--entities',
            action='store',
            dest='entities',
            type=lambda e: e.split(','),
            help=entities_help_text,
        )

        parser.add_argument(
            '--period_started_at',
            action='store',
            dest='period_started_at',
            type=lambda started_at: datetime.strptime(started_at, DATETIME_FORMAT),
            default=datetime.combine(date.today(), time.min),
            help=(
                'Дата и время начала периода сбора записей моделей РВД. Значение предоставляется в формате '
                '"дд.мм.гггг чч:мм:сс". По умолчанию, сегодняшний день, время 00:00:00.'
            ),
        )

        parser.add_argument(
            '--period_ended_at',
            action='store',
            dest='period_ended_at',
            type=(
                lambda ended_at: datetime.strptime(ended_at, DATETIME_FORMAT).replace(microsecond=time.max.microsecond)
            ),
            default=datetime.combine(date.today(), time.max),
            help=(
                'Дата и время конца периода сбора записей моделей РВД. Значение предоставляется в формате '
                '"дд.мм.гггг чч:мм:сс". По умолчанию, сегодняшний день, время 23:59:59.'
            ),
        )
        parser.add_argument(
            '--task_id',
            action='store',
            dest='task_id',
            type=str,
            default=None,
            help='task_id для поиска асинхронной задачи',
        )
        parser.add_argument(
            '--no-update-modified',
            dest='update_modified',
            action='store_false',
            default=True,
            help='Не обновлять поле modified моделей',
        )

    def _prepare_export_entities_data_class(self, *args, **kwargs) -> BaseExportEntitiesData:
        """Возвращает объект класса экспорта данных сущностей РВД."""
        raise NotImplementedError

    def handle(self, *args, **kwargs):
        """
        Выполнение действий команды.
        """
        export_entities_data = self._prepare_export_entities_data_class(*args, **kwargs)
        export_entities_data.export()


class BaseCollectModelsDataByGeneratingLogsCommand(BaseCollectModelDataCommand):
    """
    Команда сбора данных моделей РВД на основе существующих в БД данных моделей ЭШ.

    Можно регулировать, для каких моделей должен быть произведен сбор данных, и период, за который должны
    быть собраны логи. Логи формируются в процессе выполнения команды при помощи генератора логов LogGenerator для
    указанной модели.
    """

    # flake8: noqa: A003
    help = 'Команда сбора данных моделей РВД на основе существующих в БД данных моделей продукта'

    def add_arguments(self, parser: 'CommandParser'):
        """
        Добавление параметров.
        """
        super().add_arguments(parser=parser)

        parser.add_argument(
            '--logs_sub_period_days',
            action='store',
            dest='logs_sub_period_days',
            type=int,
            default=0,
            help=(
                'Размер подпериодов, на которые будет разбит основной период, в днях. По умолчанию, '
                '0 - разбиение на подпериоды отключено.'
            ),
        )

        parser.add_argument(
            '--school_ids',
            action='store',
            dest='school_ids',
            type=lambda v: tuple(map(int, v.split(','))),
            default=(),
            help='Школы, для которых производится выгрузка.',
        )



from abc import (
    ABCMeta,
)

from function_tools.validators import (
    BaseValidator,
)


class WebEduRunnerValidator(BaseValidator, metaclass=ABCMeta):
    """
    Базовый класс валидаторов ранеров функций продуктов Образования.
    """


class WebEduFunctionValidator(BaseValidator, metaclass=ABCMeta):
    """
    Базовый класс валидаторов функций продуктов Образования.
    """
